<?php
/*--------------------------------------------+
| Teamspeak2 Viewer     V2.1     2005-05-11   |
|---------------------------------------------|
| author: Makro (Markus Kronier)        2005 |
| web: http://www.makromobil.de               |
| email: makro@makromobil.de                  |
|---------------------------------------------|
| Released under the terms and conditions of  |
| the GNU General Public License (Version 2)  |
+--------------------------------------------*/

  //---- Sortcompare function
  function ChannelSortCmp($a, $b) {
    if ($a["order"] == $b["order"]) {
      return strcasecmp($a["name"],$b["name"]);
    }

    return (intval($a["order"]) < intval($b["order"]))? -1: 1;
  }

  class TSViewer {
    //--- Default Settings
    var $tsviewerId	      = 0;
    var $serverIp         = "127.0.0.1";
    var $queryPort        = "51234";
    var $udpPort          = "8767";
    var $LoginName        = "Guest";
    var $LoginPwd         = "";
    var $connectLink      = 0;
    var $hideEmptyChannels= 0;
    var $infoLink	      = 1;
    var $enabled          = 1;
    var $baseDir          = "";

    //---------------------------------------------------
    //---- NO CHANGES BEHIND THIS LINE
    var $socket;
    var $serverInfo      = array();
    var $channelList     = array();
    var $errors	         = array("1"=>"No TS-Server running","2"=>"not connected to the Server");
    var $errorMsg        = "";
    var $serverVersion   = "unknown";

    //--- Codec ausgeben
    function getCodec($nCodec) {
      $Codec = "unknown";
      switch ($nCodec) {
        case 0:  $Codec = "CELP 5.1 Kbit";   break;
        case 1:  $Codec = "CELP 6.3 Kbit";   break;
        case 2:  $Codec = "GSM 14.8 Kbit";   break;
        case 3:  $Codec = "GSM 16.4 Kbit";   break;
        case 4:  $Codec = "CELP Windows 5.2 Kbit"; break;
        case 5:  $Codec = "Speex 3.4 Kbit";  break;
        case 6:  $Codec = "Speex 5.2 Kbit";  break;
        case 7:  $Codec = "Speex 7.2 Kbit";  break;
        case 8:  $Codec = "Speex 9.3 Kbit";  break;
        case 9:  $Codec = "Speex 12.3 Kbit"; break;
        case 10: $Codec = "Speex 16.3 Kbit"; break;
        case 11: $Codec = "Speex 19.5 Kbit"; break;
        case 12: $Codec = "Speex 25.9 Kbit"; break;
      }
      return $Codec;
    }

    //--- Are Player in Channel/Subchannel
    function PlayerInChannel($Channel) {
      if (count($Channel["PlayerList"]) > 0)
        return true;
      foreach($Channel["ChannelList"] as $sChannel) {
        if ($this->PlayerInChannel($sChannel)) {
          return true;
        }
      }
      return false;
    }

    //--- Check if Channel is Defaultchannel
    function IsDefaultChannel($Channel) {
      if (array_key_exists("flags", $Channel)) {
        $flag = $Channel["flags"];
        if (($flag & 0x10) != 0)
        {
          return true;
        }
      }
      return false;
    }

    //--- Insert Channel in ChannelList
    function InsertChannel(&$Channel) {
      $nListCount = count($this->channelList);
      for ($nChannel= --$nListCount; $nChannel >=0; $nChannel--) {
        if ($this->channelList[$nChannel]["parent"] == $Channel["id"]) {
          array_push($Channel["ChannelList"], $this->channelList[$nChannel]);
          unset($this->channelList[$nChannel]);
        }
      }
      $this->channelList = array_values($this->channelList);
      $nListCount = count($this->channelList);
      for ($nChannel= --$nListCount; $nChannel >=0; $nChannel--) {
        if ($this->channelList[$nChannel]["id"] == $Channel["parent"]) {
          array_push($this->channelList[$nChannel]["ChannelList"], $Channel);
          unset($Channel);
        }
      }
      if (isset($Channel)) {
        array_push($this->channelList, $Channel);
       }
      return true;
    }

    //--- Insert Player in his current Channel-------------------------
    function InsertPlayer(&$lstChannels, $Player) {
      for ($nChannel=0; $nChannel < count($lstChannels); $nChannel++) {
        if ($lstChannels[$nChannel]["id"] == $Player["c_id"]) {
          array_push($lstChannels[$nChannel]["PlayerList"], $Player);
          return true;
        }
      }
      for ($nChannel=0; $nChannel < count($lstChannels); $nChannel++) {
        if ($this->InsertPlayer($lstChannels[$nChannel]["ChannelList"], $Player)) {
          return true;
        }
      }
      return false;
    }

    //--- Sortfkt for Sorting Channels
    function ChannelSort(&$ChannelList) {
      usort($ChannelList, ChannelSortCmp);
      for ($ch=0;$ch < count($ChannelList); $ch++) {
        $this->ChannelSort($ChannelList[$ch]["ChannelList"]);
      }
    }

    //--- Read Serverdata--------------------------------
    function ReadServerData() {
      //--- Message-termination
      $CRLF = chr(13).chr(10);

      //--- Clear Data
      unset($this->socket);
      unset($this->serverInfo);
      unset($this->channelList);
      unset($this->serverVersion);

      $this->serverInfo     = array();
      $this->channelList    = array();
      $this->serverVersion  = "unknown";
      $this->errorMsg       = "";

      if (!$this->enabled) return true;

      //--- No errors!
      error_reporting (0);

      //--- Socket ffnen
      $this->socket = fsockopen($this->serverIp, $this->queryPort, $errNo, $errStr, 3);
      if($this->socket) {  //--- Connected
        //--- Timeout 3 sec
        socket_set_timeout($this->socket, 3);

        $TS = strtok(fgets($this->socket),$CRLF);
        if ($TS == "[TS]") { //-- Kennung TS vom Server erhalten!
          //---  Read Version
          fputs($this->socket, "ver".$CRLF);
          $this->serverVersion = strtok(fgets($this->socket),$CRLF);
          $OK = strtok(fgets($this->socket),$CRLF);

          //--- Select Server
          fputs($this->socket, "sel ".$this->udpPort.$CRLF);
          $OK = strtok(fgets($this->socket),$CRLF);

          //--- Get Serverinfo
          fputs($this->socket, "si ".$this->udpPort.$CRLF);
          do {
            $InfoKey = strtok(strtok(fgets($this->socket),$CRLF),"=");
            $Info    = strtok("=");
            if ($InfoKey != "OK") {
              $this->serverInfo[$InfoKey]=$Info;
            }
          } while (($InfoKey != "OK") && ($InfoKey != ""));

          //--- Get Channellist
          fputs($this->socket, "cl ".$this->udpPort.$CRLF);
          $channelFields = explode("\t", strtok(fgets($this->socket), $CRLF));

          do {
            $channelValues = explode("\t", strtok(fgets($this->socket), $CRLF));

            if (count($channelValues) >= count($channelFields)) {
              for ($Field = 0; $Field < count($channelFields); $Field++){
                $Channel[$channelFields[$Field]] = str_replace(array("\"","<",">","?"), array(""), $channelValues[$Field]);
              }
              $Channel["ChannelList"]= array();
              $Channel["PlayerList"] = array();
              $this->InsertChannel($Channel);
            } else {
              break;  //--- Got OK-message!
            }
          } while (true);

          //--- Get Playerlist
          fputs($this->socket, "pl ".$this->udpPort.$CRLF);
          $playerFields = explode("\t", strtok(fgets($this->socket), $CRLF));

          do {
            $playerValues = explode("\t", strtok(fgets($this->socket), $CRLF));
            if (count($playerValues) >= count($playerFields)) {
              for ($Field = 0; $Field < count($playerFields); $Field++){
                $Player[$playerFields[$Field]] = str_replace(array("\"","<",">","?"), array(""),$playerValues[$Field]);
              }
              $this->InsertPlayer($this->channelList, $Player);
            } else {
              break;
            }
          } while (true); //--- Got OK-message!

          //--- Send quit and disconnect
          fputs($this->socket, "quit");
          $this->errorMsg ="";
        } else {
          $this->errorMsg = $this->errors["1"]; //"No TS-Server running";
        }
        fclose($this->socket);
      } else {
        $this->errorMsg =$this->errors["2"]; //"not connected to the Server";
      }
      //--- Sort Channels in List
      $this->ChannelSort($this->channelList);

      return true;
    }

    //--- output html for Playerdata
    function GetPlayerData($playerList, $offset) {
      $Out="";
      foreach($playerList as $Player) {
        //---
        $playerIcon = "player.gif";
        if (array_key_exists("pflags", $Player)) {
          $privs = $Player["pflags"];
          if ($privs & 0x01) $playerIcon = "channelcommander.gif";    // Channel Commander
          if ($privs & 0x02) $playerIcon = "player.gif";              // Voice Request
          if ($privs & 0x04) $playerIcon = "player.gif";              // Block Whispers
          if ($privs & 0x08) $playerIcon = "away.gif";                // Away
          if ($privs & 0x10) $playerIcon = "mutemicro.gif";           // Mute Microphone
          if ($privs & 0x20) $playerIcon = "mutespeakers.gif";        // Mute Speakers
          if ($privs & 0x40) $playerIcon = "record.gif";              // Recording
        }
        $playerPrivs = "";
        if (array_key_exists("cprivs", $Player)) {
          $privs = $Player["cprivs"];
          if ($privs & 0x01) $playerPrivs .= "CA ";    // Channel Admin
          if ($privs & 0x02) $playerPrivs .= "O ";     // Operator
          if ($privs & 0x04) $playerPrivs .= "V ";     // Voice
          if ($privs & 0x08) $playerPrivs .= "AO ";    // AutoOperator
          if ($privs & 0x10) $playerPrivs .= "AV ";    // AutoVoice
        }
        if (array_key_exists("pprivs", $Player)) {
          $privs = $Player["pprivs"];
          if ($privs & 0x01) $playerPrivs .= "SA ";    // ServerAdmin
          if ($privs & 0x02) $playerPrivs .= "r";      // Allow Registration
          if ($privs & 0x04) $playerPrivs .= "R "; else $playerPrivs .= "U ";   // Registered
          //if ($privs & 0x08) $playerPrivs .= "";       // Internal Use
          //if ($privs & 0x10) $playerPrivs .= "";       // Stickey
        }
        $Out .= "<tr class=\"tsvplayer\" height=\"16\"><td class=\"tsvplayer\">\n";
        for ($nGitter=0; $nGitter < $offset; $nGitter++) {
          $Out .="<img src=\"".$this->baseDir."images/gitter.gif\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">";
        }
        $Out .="<img src=\"".$this->baseDir."images/gitter2.gif\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">";
        $Out .="<img src=\"".$this->baseDir."images/".$playerIcon."\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">";

        if ($this->infoLink) {
          $Out .="<a class=\"tsvlink\" href=\"".$_SERVER['PHP_SELF']."?tsId=".$this->tsviewerId."&p_id=".$Player["p_id"]."\">".$Player["nick"]."</a> ".$playerPrivs;
        } else {
          $Out .=$Player["nick"]." ".$playerPrivs;
        }
        $Out .="</td></tr>\n";
      }
      return $Out;
    }

    //--- output html for Channeldata
    function GetChannelData($channelList, $offset) {
      $Out="";
      foreach($channelList as $Channel) {
        if (($this->hideEmptyChannels == "0") || ($this->IsDefaultChannel($Channel)) || ($this->PlayerInChannel($Channel)) )
        {
          $channelIcon = "channel.gif";
          if (array_key_exists("flags", $Channel)) {
            $flag = $Channel["flags"];
            if ($flag & 0x01) $channelIcon = "channel.gif";    // Unregistered
            if ($flag & 0x02) $channelIcon = "channel.gif";    // Moderated
            if ($flag & 0x08) $channelIcon = "channel.gif";    // Registered + Subchannel
            if ($flag & 0x04) $channelIcon = "channelpw.gif";  // Moderated + Password
            if ($flag & 0x10) $channelIcon = "channeldef.gif"; // Registered + Default
          }
          //--- Display Channelrow
          $Out .= "<tr class=\"tsvchannel\" height=\"16\"><td class=\"tsvchannel\">\n";
          for ($nGitter=0; $nGitter < $offset; $nGitter++) {
            $Out .="<img src=\"".$this->baseDir."images/gitter.gif\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">";
          }
          $Out .="<img src=\"".$this->baseDir."images/gitter2.gif\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">";
          $Out .="<img src=\"".$this->baseDir."images/".$channelIcon."\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">";
          if ($this->infoLink) {
            $Out .= "<a class=\"tsvlink\" href=\"".$_SERVER['PHP_SELF']."?tsId=".$this->tsviewerId."&c_id=".$Channel["id"]."\">".$Channel["name"]."</a> ".$playerPrivs;
          } else {
            if ($this->connectLink) {
              $Out .= "<a class=\"tsvlink\" href=\"teamspeak://".$this->serverIp.":".$this->udpPort."/?channel=".$Channel["name"]."?password=".$this->LoginPwd."?nickname=".$this->LoginName."\">".$Channel["name"]."</a>";
            } else {
              $Out .= $Channel["name"];
            }
          }

          $Out .="</td></tr>\n";
          //--- Display all Players of current Channel
          $Out .= $this->GetPlayerData($Channel["PlayerList"],$offset+1);

          //--- Display ChildChannels
          $Out .= $this->GetChannelData($Channel["ChannelList"], $offset+1);
        }
      }
      return $Out;
    }

    function GetServerParams() {
      $Out = "";
      foreach($this->serverInfo as $Info => $Value) {
        $Out .= $Info ." ".$Value."<br><br>\n";
      }
      return $Out;
    }

    function GetServerInfo($serverId) {
      if ($this->serverInfo["server_id"]==$serverId) {
        if (isset($this->serverInfo["server_name"])) {
          $Out  = "<b>Server:</b><br>".$this->serverInfo["server_name"]."<br><br>\n";
          $Out .= "<b>Server IP:</b><br>".$this->serverIp.":".$this->serverInfo["server_udpport"]."<br><br>\n";
          $Out .= "<b>Version:</b><br>".$this->serverVersion."<br><br>\n";
          $Out .= "<b>Platform:</b><br>".$this->serverInfo["server_platform"]."<br><br>\n";
          $Out .= "<b>User:</b><br> (".$this->serverInfo["server_currentusers"]." / ".$this->serverInfo["server_maxusers"].")<br><br>\n";
          $Out .= "<b>Welcome Message:</b><br>".$this->serverInfo["server_welcomemessage"]."<br><br>\n";
        }
      }
      if ($this->connectLink) {
        $Out .= "<a class=\"tsvlink\" href=\"teamspeak://".$this->serverIp.":".$this->udpPort."/?password=".$this->LoginPwd."?nickname=".$this->LoginName."\">connect...</a><br><br>";
      }
      return $Out;
    }

    function GetChannelInfo($lstChannels, $channelId) {
      $Out = "";
      foreach($lstChannels as $Channel) {
        if ($Channel["id"]==$channelId) {
          $Out  = "<b>Channel:</b><br>".$Channel["name"]."<br><br>";
          $Out .= "<b>Topic:</b><br>".$Channel["topic"]."<br><br>";
          $Out .= "<b>User in channel:</b><br>(".count($Channel["PlayerList"])."/".$Channel["maxusers"].")<br><br>";
          $Out .= "<b>Codec:</b><br>".$this->getCodec($Channel["codec"])."<br><br>";
          $Out .= "<b>Description:</b><br><br>";
          if ($this->connectLink) {
            $Out .= "<a class=\"tsvlink\" href=\"teamspeak://".$this->serverIp.":".$this->udpPort."/?channel=".$Channel["name"]."?password=".$this->LoginPwd."?nickname=".$this->LoginName."\">connect...</a><br><br>";
          }
        } else {
          if (count($Channel["ChannelList"])) {
            $Out = $this->GetChannelInfo($Channel["ChannelList"], $channelId);
          }
        }
        if ($Out != "") return $Out;
      }
      return $Out;
    }

    function GetPlayerInfo($lstChannels, $playerId) {
      $Out = "";
      foreach($lstChannels as $Channel) {
        foreach ($Channel["PlayerList"] as $Player) {
          if ($Player["p_id"]==$playerId) {
            $Out = "<b>Player:</b><br>".$Player["nick"]."<br><br>";
            $Out .= "<b>Global flags:</b><br>";

            if (array_key_exists("pprivs", $Player)) {
              $privs = $Player["pprivs"];
              if ($privs & 0x01) $Out .= "ServerAdmin<br>";
              if ($privs & 0x02) $Out .= "Registration is allowed<br>";
              if ($privs & 0x04) $Out .= "Registered<br>";
            }
            $Out .= "<br><br>";
            $Out .= "<b>Channel Privilegs:</b><br>";
            if (array_key_exists("cprivs", $Player)) {
              $privs = $Player["cprivs"];
              if ($privs & 0x01) $Out .= "Channel Admin CA<br>";    // Channel Admin
              if ($privs & 0x02) $Out .= "Operator<br>";            // Operator
              if ($privs & 0x04) $Out .= "Voice<br>";               // Voice
              if ($privs & 0x08) $Out .= "AutoOperator<br>";        // AutoOperator
              if ($privs & 0x10) $Out .= "AutoVoice<br>";           // AutoVoice
              if ($privs ==0)    $Out .= "None<br>";
            }
            $Out .= "<br><br>";

            $Out .= "<b>Player Attributes:</b><br>";
            if (array_key_exists("pflags", $Player)) {
              $privs = $Player["pflags"];
              if ($privs & 0x01) $Out .= "Channel Commander<br>";
              if ($privs & 0x02) $Out .= "Voice Request<br>";
              if ($privs & 0x04) $Out .= "Does not accept whispers<br>";
              if ($privs & 0x08) $Out .= "Player is away<br>";
              if ($privs & 0x10) $Out .= "Microphone muted<br>";
              if ($privs & 0x20) $Out .= "Speaker/Headphone muted<br>";
              if ($privs & 0x40) $Out .= "Recording<br>";
              if ($privs ==0)    $Out .= "None<br>";
            }
            $Out .= "<br><br>";
          }
        }
        if (($Out =="") and (count($Channel["ChannelList"]))) {
          $Out = $this->GetPlayerInfo($Channel["ChannelList"], $playerId);
        }
        if ($Out != "") return $Out;
      }
      return $Out;
    }

    //--- get html for TS-Viewer compact Box
    function GetTSViewerCompactHtml() {
      $this->infoLink = 0;
      $Out =  "<table class=\"tsviewer\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n";
              if ($this->errorMsg !="") {
                $Out .= "<tr class=\"tsvtitle\"><td class=\"tsvtitle\">".$this->errorMsg."</td></tr>";
              } else {
               //--- Refresh
                $Out .= "<tr class=\"tsvchannel\"><td class=\"tsvchannel\">";
                $Out .= "<a href=\"".$_SERVER['PHP_SELF']."?tsId=".$this->tsviewerId."\"><img src=\"".$this->baseDir."images/refresh.gif\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">refresh</a>";
                $Out .= "</td></tr>";

                //--- ServerInfo
                $Out .= "<tr class=\"tsvchannel\" height=\"16\"><td class=\"tsvchannel\">\n";
                $Out .= "<img src=\"".$this->baseDir."images/teamspeak.gif\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">";

                if ($this->infoLink) {
                  $Out .= "<a class=\"tsvlink\" href=\"".$_SERVER['PHP_SELF']."?tsId=".$this->tsviewerId."&s_id=".$this->serverInfo["server_id"]."\">".$this->serverInfo["server_name"]."</a>";
                } else {
                  $Out .= $this->serverInfo["server_name"];
                }
                $Out .= "<br>";
                $Out .= "</td></tr>\n";
                //--- Display all Channels
                $Out .= $this->GetChannelData($this->channelList,0);
              }
      $Out .= "<tr class=\"tsvtitle\"><td class=\"tsvtitle\">Teamspeak2 Viewer<br> 2005 <a href=\"mailto:makro@makromobil.de\">Makro</a></td></tr>\n";
      $Out .= "</table>\n";
      return $Out;
    }

    //--- get html for TS-Viewer complete informations
    function GetTSViewerHtml() {
      $this->infoLink = 1;
      $Out  = "<table class=\"tsviewer\" width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n";
      $Out .= "<tr class=\"tsvinfo\"><td class=\"tsvinfo\" width=\"50%\" valign=\"top\">\n";
      $Out .= "<table class=\"tsviewer\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n";
              if ($this->errorMsg !="") {
                $Out .= "<tr class=\"tsvtitle\"><td class=\"tsvtitle\">".$this->errorMsg."</td></tr>";
              } else {
                //--- Refresh
                $Out .= "<tr class=\"tsvchannel\"><td class=\"tsvchannel\">";
                $Out .= "<a href=\"".$_SERVER['PHP_SELF']."?tsId=".$this->tsviewerId."\"><img src=\"".$this->baseDir."images/refresh.gif\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">refresh</a>";
                $Out .= "</td></tr>";

                //--- ServerInfo
                $Out .= "<tr class=\"tsvchannel\" height=\"16\"><td class=\"tsvchannel\">\n";
                $Out .= "<img src=\"".$this->baseDir."images/teamspeak.gif\" width=\"16\" height=\"16\" align=\"left\" border=\"0\" alt=\"\">";
                if ($this->infoLink) {
                  $Out .= "<a class=\"tsvlink\" href=\"".$_SERVER['PHP_SELF']."?tsId=".$this->tsviewerId."&s_id=".$this->serverInfo["server_id"]."\">".$this->serverInfo["server_name"]."</a>";
                } else {
                  $Out .= $this->serverInfo["server_name"];
                }
                $Out .= "</td></tr>\n";

                //--- Display all Channels
                $Out .= $this->GetChannelData($this->channelList,0);
              }
      $Out .= "<tr class=\"tsvtitle\"><td class=\"tsvtitle\">Teamspeak2 Viewer<br> 2005 <a class=\"tsvlink\" href=\"mailto:makro@makromobil.de\">Makro</a></td></tr>\n";
      $Out .= "</table>\n";

      $Out .= "</td><td class=\"tsvinfo\" width=\"50%\" valign=\"top\">\n";

      //-- Show Serverinfo
      if (isset($_GET["s_id"])) {
        $Out .= $this->GetServerInfo($_GET["s_id"]);
      } else
      //-- Show ChannelInfo
      if (isset($_GET["c_id"])) {
        $Out .= $this->GetChannelInfo($this->channelList, $_GET["c_id"]);
      } else
      if (isset($_GET["p_id"])) {
        $Out .= $this->GetPlayerInfo($this->channelList, $_GET["p_id"]);
      } else {
        $Out .= $this->GetServerInfo($this->serverInfo["server_id"]);
      }

      $Out .= "</td></tr>\n</table>\n";
      return $Out;
    }
  }

$tsviewer = new TSViewer;

?>